@@ -1,25 +1,32 @@
-import { Matcher, render, waitFor } from '@testing-library/react';
-import { Provider } from 'react-redux';
-import { BackendSrv, locationService, setBackendSrv, setDataSourceSrv } from '@grafana/runtime';
-import { configureStore } from 'app/store/configureStore';
-import RuleEditor from './RuleEditor';
-import { Route, Router } from 'react-router-dom';
+import { Matcher, render, waitFor, screen, within } from '@testing-library/react';
+import userEvent, { PointerEventsCheckLevel } from '@testing-library/user-event';
 import React from 'react';
+import { Provider } from 'react-redux';
+import { Route, Router } from 'react-router-dom';
+import { selectOptionInTest } from 'test/helpers/selectOptionInTest';
 import { byLabelText, byRole, byTestId, byText } from 'testing-library-selector';
-import { selectOptionInTest } from '@grafana/ui';
-import { contextSrv } from 'app/core/services/context_srv';
-import { mockDataSource, MockDataSourceSrv } from './mocks';
-import userEvent from '@testing-library/user-event';
+
 import { DataSourceInstanceSettings } from '@grafana/data';
-import { typeAsJestMock } from 'test/helpers/typeAsJestMock';
-import { getAllDataSources } from './utils/config';
-import { fetchRulerRules, fetchRulerRulesGroup, fetchRulerRulesNamespace, setRulerRuleGroup } from './api/ruler';
-import { DataSourceType, GRAFANA_RULES_SOURCE_NAME } from './utils/datasource';
+import { selectors } from '@grafana/e2e-selectors';
+import { locationService, setDataSourceSrv } from '@grafana/runtime';
+import { ADD_NEW_FOLER_OPTION } from 'app/core/components/Select/FolderPicker';
+import { contextSrv } from 'app/core/services/context_srv';
 import { DashboardSearchHit } from 'app/features/search/types';
-import { getDefaultQueries } from './utils/rule-form';
+import { configureStore } from 'app/store/configureStore';
+import { GrafanaAlertStateDecision, PromApplication } from 'app/types/unified-alerting-dto';
+
+import { searchFolders } from '../../../../app/features/manage-dashboards/state/actions';
+import { backendSrv } from '../../../core/services/backend_srv';
+import { AccessControlAction } from '../../../types';
+
+import RuleEditor from './RuleEditor';
+import { discoverFeatures } from './api/buildInfo';
+import { fetchRulerRules, fetchRulerRulesGroup, fetchRulerRulesNamespace, setRulerRuleGroup } from './api/ruler';
 import { ExpressionEditorProps } from './components/rule-editor/ExpressionEditor';
-import * as api from 'app/features/manage-dashboards/state/actions';
-import { GrafanaAlertStateDecision } from 'app/types/unified-alerting-dto';
+import { disableRBAC, mockDataSource, MockDataSourceSrv, mockFolder } from './mocks';
+import * as config from './utils/config';
+import { DataSourceType, GRAFANA_RULES_SOURCE_NAME } from './utils/datasource';
+import { getDefaultQueries } from './utils/rule-form';
 
 jest.mock('./components/rule-editor/ExpressionEditor', () => ({
   // eslint-disable-next-line react/display-name
@@ -28,8 +35,9 @@ jest.mock('./components/rule-editor/ExpressionEditor', () => ({
   ),
 }));
 
+jest.mock('./api/buildInfo');
 jest.mock('./api/ruler');
-jest.mock('./utils/config');
+jest.mock('../../../../app/features/manage-dashboards/state/actions');
 
 // there's no angular scope in test and things go terribly wrong when trying to render the query editor row.
 // lets just skip it
@@ -38,14 +46,17 @@ jest.mock('app/features/query/components/QueryEditorRow', () => ({
   QueryEditorRow: () => <p>hi</p>,
 }));
 
-const mocks = {
-  getAllDataSources: typeAsJestMock(getAllDataSources),
+jest.spyOn(config, 'getAllDataSources');
 
+const mocks = {
+  getAllDataSources: jest.mocked(config.getAllDataSources),
+  searchFolders: jest.mocked(searchFolders),
   api: {
-    fetchRulerRulesGroup: typeAsJestMock(fetchRulerRulesGroup),
-    setRulerRuleGroup: typeAsJestMock(setRulerRuleGroup),
-    fetchRulerRulesNamespace: typeAsJestMock(fetchRulerRulesNamespace),
-    fetchRulerRules: typeAsJestMock(fetchRulerRules),
+    discoverFeatures: jest.mocked(discoverFeatures),
+    fetchRulerRulesGroup: jest.mocked(fetchRulerRulesGroup),
+    setRulerRuleGroup: jest.mocked(setRulerRuleGroup),
+    fetchRulerRulesNamespace: jest.mocked(fetchRulerRulesNamespace),
+    fetchRulerRules: jest.mocked(fetchRulerRules),
   },
 };
 
@@ -69,6 +80,7 @@ const ui = {
     alertType: byTestId('alert-type-picker'),
     dataSource: byTestId('datasource-picker'),
     folder: byTestId('folder-picker'),
+    folderContainer: byTestId(selectors.components.FolderPicker.containerV2),
     namespace: byTestId('namespace-picker'),
     group: byTestId('group-picker'),
     annotationKey: (idx: number) => byTestId(`annotation-key-${idx}`),
@@ -81,15 +93,22 @@ const ui = {
     save: byRole('button', { name: 'Save' }),
     addAnnotation: byRole('button', { name: /Add info/ }),
     addLabel: byRole('button', { name: /Add label/ }),
+    // alert type buttons
+    grafanaManagedAlert: byRole('button', { name: /Grafana managed/ }),
+    lotexAlert: byRole('button', { name: /Mimir or Loki alert/ }),
+    lotexRecordingRule: byRole('button', { name: /Mimir or Loki recording rule/ }),
   },
 };
 
 describe('RuleEditor', () => {
   beforeEach(() => {
-    jest.resetAllMocks();
+    jest.clearAllMocks();
     contextSrv.isEditor = true;
+    contextSrv.hasEditPermissionInFolders = true;
   });
 
+  disableRBAC();
+
   it('can create a new cloud alert', async () => {
     const dataSources = {
       default: mockDataSource(
@@ -124,23 +143,37 @@ describe('RuleEditor', () => {
         },
       ],
     });
+    mocks.searchFolders.mockResolvedValue([]);
+
+    mocks.api.discoverFeatures.mockResolvedValue({
+      application: PromApplication.Lotex,
+      features: {
+        rulerApiEnabled: true,
+      },
+    });
 
     await renderRuleEditor();
-    await userEvent.type(await ui.inputs.name.find(), 'my great new rule');
-    await clickSelectOption(ui.inputs.alertType.get(), /Cortex\/Loki managed alert/);
+    await waitFor(() => expect(mocks.searchFolders).toHaveBeenCalled());
+    await waitFor(() => expect(mocks.api.discoverFeatures).toHaveBeenCalled());
+
+    await userEvent.click(await ui.buttons.lotexAlert.find());
+
     const dataSourceSelect = ui.inputs.dataSource.get();
-    userEvent.click(byRole('textbox').get(dataSourceSelect));
+    await userEvent.click(byRole('combobox').get(dataSourceSelect));
     await clickSelectOption(dataSourceSelect, 'Prom (default)');
     await waitFor(() => expect(mocks.api.fetchRulerRules).toHaveBeenCalled());
+
+    await userEvent.type(await ui.inputs.expr.find(), 'up == 1');
+
+    await userEvent.type(ui.inputs.name.get(), 'my great new rule');
     await clickSelectOption(ui.inputs.namespace.get(), 'namespace2');
     await clickSelectOption(ui.inputs.group.get(), 'group2');
 
-    await userEvent.type(ui.inputs.expr.get(), 'up == 1');
-
     await userEvent.type(ui.inputs.annotationValue(0).get(), 'some summary');
     await userEvent.type(ui.inputs.annotationValue(1).get(), 'some description');
 
-    userEvent.click(ui.buttons.addLabel.get());
+    // TODO remove skipPointerEventsCheck once https://github.com/jsdom/jsdom/issues/3232 is fixed
+    await userEvent.click(ui.buttons.addLabel.get(), { pointerEventsCheck: PointerEventsCheckLevel.Never });
 
     await userEvent.type(ui.inputs.labelKey(0).get(), 'severity');
     await userEvent.type(ui.inputs.labelValue(0).get(), 'warn');
@@ -148,24 +181,61 @@ describe('RuleEditor', () => {
     await userEvent.type(ui.inputs.labelValue(1).get(), 'the a-team');
 
     // save and check what was sent to backend
-    userEvent.click(ui.buttons.save.get());
+    await userEvent.click(ui.buttons.save.get());
     await waitFor(() => expect(mocks.api.setRulerRuleGroup).toHaveBeenCalled());
-    expect(mocks.api.setRulerRuleGroup).toHaveBeenCalledWith('Prom', 'namespace2', {
-      name: 'group2',
-      rules: [
-        {
-          alert: 'my great new rule',
-          annotations: { description: 'some description', summary: 'some summary' },
-          labels: { severity: 'warn', team: 'the a-team' },
-          expr: 'up == 1',
-          for: '1m',
-        },
-      ],
-    });
+    expect(mocks.api.setRulerRuleGroup).toHaveBeenCalledWith(
+      { dataSourceName: 'Prom', apiVersion: 'legacy' },
+      'namespace2',
+      {
+        name: 'group2',
+        rules: [
+          {
+            alert: 'my great new rule',
+            annotations: { description: 'some description', summary: 'some summary' },
+            labels: { severity: 'warn', team: 'the a-team' },
+            expr: 'up == 1',
+            for: '1m',
+          },
+        ],
+      }
+    );
   });
 
   it('can create new grafana managed alert', async () => {
-    const searchFolderMock = jest.spyOn(api, 'searchFolders').mockResolvedValue([
+    const dataSources = {
+      default: mockDataSource(
+        {
+          type: 'prometheus',
+          name: 'Prom',
+          isDefault: true,
+        },
+        { alerting: true }
+      ),
+    };
+
+    setDataSourceSrv(new MockDataSourceSrv(dataSources));
+    mocks.getAllDataSources.mockReturnValue(Object.values(dataSources));
+    mocks.api.setRulerRuleGroup.mockResolvedValue();
+    mocks.api.fetchRulerRulesNamespace.mockResolvedValue([]);
+    mocks.api.fetchRulerRulesGroup.mockResolvedValue({
+      name: 'group2',
+      rules: [],
+    });
+    mocks.api.fetchRulerRules.mockResolvedValue({
+      namespace1: [
+        {
+          name: 'group1',
+          rules: [],
+        },
+      ],
+      namespace2: [
+        {
+          name: 'group2',
+          rules: [],
+        },
+      ],
+    });
+    mocks.searchFolders.mockResolvedValue([
       {
         title: 'Folder A',
         id: 1,
@@ -174,32 +244,37 @@ describe('RuleEditor', () => {
         title: 'Folder B',
         id: 2,
       },
+      {
+        title: 'Folder / with slash',
+        id: 2,
+      },
     ] as DashboardSearchHit[]);
 
-    const dataSources = {
-      default: mockDataSource({
-        type: 'prometheus',
-        name: 'Prom',
-        isDefault: true,
-      }),
-    };
-
-    setDataSourceSrv(new MockDataSourceSrv(dataSources));
-    mocks.api.setRulerRuleGroup.mockResolvedValue();
-    mocks.api.fetchRulerRulesNamespace.mockResolvedValue([]);
+    mocks.api.discoverFeatures.mockResolvedValue({
+      application: PromApplication.Prometheus,
+      features: {
+        rulerApiEnabled: false,
+      },
+    });
 
     // fill out the form
     await renderRuleEditor();
-    userEvent.type(await ui.inputs.name.find(), 'my great new rule');
-    await clickSelectOption(ui.inputs.alertType.get(), /Classic Grafana alerts based on thresholds/);
+    await waitFor(() => expect(mocks.searchFolders).toHaveBeenCalled());
+    await waitFor(() => expect(mocks.api.discoverFeatures).toHaveBeenCalled());
+
+    await userEvent.type(await ui.inputs.name.find(), 'my great new rule');
+
     const folderInput = await ui.inputs.folder.find();
-    await waitFor(() => expect(searchFolderMock).toHaveBeenCalled());
     await clickSelectOption(folderInput, 'Folder A');
 
+    const groupInput = screen.getByRole('textbox', { name: /^Group/ });
+    await userEvent.type(groupInput, 'my group');
+
     await userEvent.type(ui.inputs.annotationValue(0).get(), 'some summary');
     await userEvent.type(ui.inputs.annotationValue(1).get(), 'some description');
 
-    userEvent.click(ui.buttons.addLabel.get());
+    // TODO remove skipPointerEventsCheck once https://github.com/jsdom/jsdom/issues/3232 is fixed
+    await userEvent.click(ui.buttons.addLabel.get(), { pointerEventsCheck: PointerEventsCheckLevel.Never });
 
     await userEvent.type(ui.inputs.labelKey(0).get(), 'severity');
     await userEvent.type(ui.inputs.labelValue(0).get(), 'warn');
@@ -207,26 +282,30 @@ describe('RuleEditor', () => {
     await userEvent.type(ui.inputs.labelValue(1).get(), 'the a-team');
 
     // save and check what was sent to backend
-    userEvent.click(ui.buttons.save.get());
+    await userEvent.click(ui.buttons.save.get());
     await waitFor(() => expect(mocks.api.setRulerRuleGroup).toHaveBeenCalled());
-    expect(mocks.api.setRulerRuleGroup).toHaveBeenCalledWith(GRAFANA_RULES_SOURCE_NAME, 'Folder A', {
-      interval: '1m',
-      name: 'my great new rule',
-      rules: [
-        {
-          annotations: { description: 'some description', summary: 'some summary' },
-          labels: { severity: 'warn', team: 'the a-team' },
-          for: '5m',
-          grafana_alert: {
-            condition: 'B',
-            data: getDefaultQueries(),
-            exec_err_state: 'Alerting',
-            no_data_state: 'NoData',
-            title: 'my great new rule',
+    expect(mocks.api.setRulerRuleGroup).toHaveBeenCalledWith(
+      { dataSourceName: GRAFANA_RULES_SOURCE_NAME, apiVersion: 'legacy' },
+      'Folder A',
+      {
+        interval: '1m',
+        name: 'my group',
+        rules: [
+          {
+            annotations: { description: 'some description', summary: 'some summary' },
+            labels: { severity: 'warn', team: 'the a-team' },
+            for: '5m',
+            grafana_alert: {
+              condition: 'B',
+              data: getDefaultQueries(),
+              exec_err_state: GrafanaAlertStateDecision.Error,
+              no_data_state: 'NoData',
+              title: 'my great new rule',
+            },
           },
-        },
-      ],
-    });
+        ],
+      }
+    );
   });
 
   it('can create a new cloud recording rule', async () => {
@@ -263,50 +342,69 @@ describe('RuleEditor', () => {
         },
       ],
     });
+    mocks.searchFolders.mockResolvedValue([]);
+
+    mocks.api.discoverFeatures.mockResolvedValue({
+      application: PromApplication.Lotex,
+      features: {
+        rulerApiEnabled: true,
+      },
+    });
 
     await renderRuleEditor();
+    await waitFor(() => expect(mocks.searchFolders).toHaveBeenCalled());
+    await waitFor(() => expect(mocks.api.discoverFeatures).toHaveBeenCalled());
     await userEvent.type(await ui.inputs.name.find(), 'my great new recording rule');
-    await clickSelectOption(ui.inputs.alertType.get(), /Cortex\/Loki managed recording rule/);
+    await userEvent.click(await ui.buttons.lotexRecordingRule.get());
+
     const dataSourceSelect = ui.inputs.dataSource.get();
-    userEvent.click(byRole('textbox').get(dataSourceSelect));
+    await userEvent.click(byRole('combobox').get(dataSourceSelect));
+
     await clickSelectOption(dataSourceSelect, 'Prom (default)');
     await waitFor(() => expect(mocks.api.fetchRulerRules).toHaveBeenCalled());
     await clickSelectOption(ui.inputs.namespace.get(), 'namespace2');
     await clickSelectOption(ui.inputs.group.get(), 'group2');
 
-    await userEvent.type(ui.inputs.expr.get(), 'up == 1');
+    await userEvent.type(await ui.inputs.expr.find(), 'up == 1');
 
-    userEvent.click(ui.buttons.addLabel.get());
+    // TODO remove skipPointerEventsCheck once https://github.com/jsdom/jsdom/issues/3232 is fixed
+    await userEvent.click(ui.buttons.addLabel.get(), { pointerEventsCheck: PointerEventsCheckLevel.Never });
 
     await userEvent.type(ui.inputs.labelKey(1).get(), 'team');
     await userEvent.type(ui.inputs.labelValue(1).get(), 'the a-team');
 
     // try to save, find out that recording rule name is invalid
-    userEvent.click(ui.buttons.save.get());
-    expect(
-      await byText(
-        'Recording rule name must be valid metric name. It may only contain letters, numbers, and colons. It may not contain whitespace.'
-      ).find()
-    ).toBeInTheDocument();
+    await userEvent.click(ui.buttons.save.get());
+    await waitFor(() =>
+      expect(
+        byText(
+          'Recording rule name must be valid metric name. It may only contain letters, numbers, and colons. It may not contain whitespace.'
+        ).get()
+      ).toBeInTheDocument()
+    );
     expect(mocks.api.setRulerRuleGroup).not.toBeCalled();
 
     // fix name and re-submit
-    await userEvent.type(await ui.inputs.name.find(), '{selectall}{del}my:great:new:recording:rule');
-    userEvent.click(ui.buttons.save.get());
+    await userEvent.clear(await ui.inputs.name.find());
+    await userEvent.type(await ui.inputs.name.find(), 'my:great:new:recording:rule');
 
     // save and check what was sent to backend
-    userEvent.click(ui.buttons.save.get());
+    await userEvent.click(ui.buttons.save.get());
     await waitFor(() => expect(mocks.api.setRulerRuleGroup).toHaveBeenCalled());
-    expect(mocks.api.setRulerRuleGroup).toHaveBeenCalledWith('Prom', 'namespace2', {
-      name: 'group2',
-      rules: [
-        {
-          record: 'my:great:new:recording:rule',
-          labels: { team: 'the a-team' },
-          expr: 'up == 1',
-        },
-      ],
-    });
+    expect(mocks.api.setRulerRuleGroup).toHaveBeenCalledWith(
+      { dataSourceName: 'Prom', apiVersion: 'legacy' },
+      'namespace2',
+      {
+        name: 'group2',
+        rules: [
+          {
+            record: 'my:great:new:recording:rule',
+            labels: { team: 'the a-team' },
+            expr: 'up == 1',
+          },
+        ],
+      }
+    );
   });
 
   it('can edit grafana managed rule', async () => {
@@ -316,24 +414,34 @@ describe('RuleEditor', () => {
       uid: 'abcd',
       id: 1,
     };
-    jest.spyOn(api, 'searchFolders').mockResolvedValue([folder] as DashboardSearchHit[]);
-    const getFolderByUid = jest.fn().mockResolvedValue({
-      ...folder,
-      canSave: true,
-    });
-    const dataSources = {
-      default: mockDataSource({
-        type: 'prometheus',
-        name: 'Prom',
-        isDefault: true,
-      }),
+
+    const slashedFolder = {
+      title: 'Folder with /',
+      uid: 'abcde',
+      id: 2,
     };
 
-    const backendSrv = ({
-      getFolderByUid,
-    } as any) as BackendSrv;
-    setBackendSrv(backendSrv);
+    const dataSources = {
+      default: mockDataSource(
+        {
+          type: 'prometheus',
+          name: 'Prom',
+          isDefault: true,
+        },
+        { alerting: true }
+      ),
+    };
+
+    jest.spyOn(backendSrv, 'getFolderByUid').mockResolvedValue({
+      ...mockFolder(),
+      accessControl: {
+        [AccessControlAction.AlertingRuleUpdate]: true,
+      },
+    });
+
     setDataSourceSrv(new MockDataSourceSrv(dataSources));
+
+    mocks.getAllDataSources.mockReturnValue(Object.values(dataSources));
     mocks.api.setRulerRuleGroup.mockResolvedValue();
     mocks.api.fetchRulerRulesNamespace.mockResolvedValue([]);
     mocks.api.fetchRulerRules.mockResolvedValue({
@@ -352,7 +460,7 @@ describe('RuleEditor', () => {
                 namespace_id: 1,
                 condition: 'B',
                 data: getDefaultQueries(),
-                exec_err_state: GrafanaAlertStateDecision.Alerting,
+                exec_err_state: GrafanaAlertStateDecision.Error,
                 no_data_state: GrafanaAlertStateDecision.NoData,
                 title: 'my great new rule',
               },
@@ -361,16 +469,33 @@ describe('RuleEditor', () => {
         },
       ],
     });
+    mocks.searchFolders.mockResolvedValue([folder, slashedFolder] as DashboardSearchHit[]);
 
     await renderRuleEditor(uid);
+    await waitFor(() => expect(mocks.searchFolders).toHaveBeenCalled());
+    await waitFor(() => expect(mocks.api.discoverFeatures).toHaveBeenCalled());
+    await waitFor(() => expect(mocks.searchFolders).toHaveBeenCalled());
 
     // check that it's filled in
     const nameInput = await ui.inputs.name.find();
     expect(nameInput).toHaveValue('my great new rule');
+    //check that folder is in the list
     expect(ui.inputs.folder.get()).toHaveTextContent(new RegExp(folder.title));
     expect(ui.inputs.annotationValue(0).get()).toHaveValue('some description');
     expect(ui.inputs.annotationValue(1).get()).toHaveValue('some summary');
 
+    //check that slashed folders are not in the list
+    expect(ui.inputs.folder.get()).toHaveTextContent(new RegExp(folder.title));
+    expect(ui.inputs.folder.get()).not.toHaveTextContent(new RegExp(slashedFolder.title));
+
+    //check that slashes warning is only shown once user search slashes
+    const folderInput = await ui.inputs.folderContainer.find();
+    expect(within(folderInput).queryByText("Folders with '/' character are not allowed.")).not.toBeInTheDocument();
+    await userEvent.type(within(folderInput).getByRole('combobox'), 'new slashed //');
+    expect(within(folderInput).getByText("Folders with '/' character are not allowed.")).toBeInTheDocument();
+    await userEvent.keyboard('{backspace} {backspace}{backspace}');
+    expect(within(folderInput).queryByText("Folders with '/' character are not allowed.")).not.toBeInTheDocument();
+
     // add an annotation
     await clickSelectOption(ui.inputs.annotationKey(2).get(), /Add new/);
     await userEvent.type(byRole('textbox').get(ui.inputs.annotationKey(2).get()), 'custom');
@@ -381,28 +506,39 @@ describe('RuleEditor', () => {
     await userEvent.type(ui.inputs.labelValue(2).get(), 'value');
 
     // save and check what was sent to backend
-    userEvent.click(ui.buttons.save.get());
+    await userEvent.click(ui.buttons.save.get());
     await waitFor(() => expect(mocks.api.setRulerRuleGroup).toHaveBeenCalled());
 
-    expect(mocks.api.setRulerRuleGroup).toHaveBeenCalledWith(GRAFANA_RULES_SOURCE_NAME, 'Folder A', {
-      interval: '1m',
-      name: 'my great new rule',
-      rules: [
-        {
-          annotations: { description: 'some description', summary: 'some summary', custom: 'value' },
-          labels: { severity: 'warn', team: 'the a-team', custom: 'value' },
-          for: '5m',
-          grafana_alert: {
-            uid,
-            condition: 'B',
-            data: getDefaultQueries(),
-            exec_err_state: 'Alerting',
-            no_data_state: 'NoData',
-            title: 'my great new rule',
+    //check that '+ Add new' option is in folders drop down even if we don't have values
+    const emptyFolderInput = await ui.inputs.folderContainer.find();
+    mocks.searchFolders.mockResolvedValue([] as DashboardSearchHit[]);
+    await renderRuleEditor(uid);
+    await userEvent.click(within(emptyFolderInput).getByRole('combobox'));
+    expect(screen.getByText(ADD_NEW_FOLER_OPTION)).toBeInTheDocument();
+
+    expect(mocks.api.setRulerRuleGroup).toHaveBeenCalledWith(
+      { dataSourceName: GRAFANA_RULES_SOURCE_NAME, apiVersion: 'legacy' },
+      'Folder A',
+      {
+        interval: '1m',
+        name: 'my great new rule',
+        rules: [
+          {
+            annotations: { description: 'some description', summary: 'some summary', custom: 'value' },
+            labels: { severity: 'warn', team: 'the a-team', custom: 'value' },
+            for: '5m',
+            grafana_alert: {
+              uid,
+              condition: 'B',
+              data: getDefaultQueries(),
+              exec_err_state: GrafanaAlertStateDecision.Error,
+              no_data_state: 'NoData',
+              title: 'my great new rule',
+            },
           },
-        },
-      ],
-    });
+        ],
+      }
+    );
   });
 
   it('for cloud alerts, should only allow to select editable rules sources', async () => {
@@ -459,7 +595,45 @@ describe('RuleEditor', () => {
       ),
     };
 
-    mocks.api.fetchRulerRulesGroup.mockImplementation(async (dataSourceName: string) => {
+    mocks.api.discoverFeatures.mockImplementation(async (dataSourceName) => {
+      if (dataSourceName === 'loki with ruler' || dataSourceName === 'cortex with ruler') {
+        return {
+          application: PromApplication.Lotex,
+          features: {
+            rulerApiEnabled: true,
+            alertManagerConfigApi: false,
+            federatedRules: false,
+            querySharding: false,
+          },
+        };
+      }
+      if (dataSourceName === 'loki with local rule store') {
+        return {
+          application: PromApplication.Lotex,
+          features: {
+            rulerApiEnabled: false,
+            alertManagerConfigApi: false,
+            federatedRules: false,
+            querySharding: false,
+          },
+        };
+      }
+      if (dataSourceName === 'cortex without ruler api') {
+        return {
+          application: PromApplication.Lotex,
+          features: {
+            rulerApiEnabled: false,
+            alertManagerConfigApi: false,
+            federatedRules: false,
+            querySharding: false,
+          },
+        };
+      }
+
+      throw new Error(`${dataSourceName} not handled`);
+    });
+
+    mocks.api.fetchRulerRulesGroup.mockImplementation(async ({ dataSourceName }) => {
       if (dataSourceName === 'loki with ruler' || dataSourceName === 'cortex with ruler') {
         return null;
       }
@@ -479,18 +653,19 @@ describe('RuleEditor', () => {
 
     setDataSourceSrv(new MockDataSourceSrv(dataSources));
     mocks.getAllDataSources.mockReturnValue(Object.values(dataSources));
+    mocks.searchFolders.mockResolvedValue([]);
 
-    // render rule editor, select cortex/loki managed alerts
+    // render rule editor, select mimir/loki managed alerts
     await renderRuleEditor();
-    await ui.inputs.name.find();
-    await clickSelectOption(ui.inputs.alertType.get(), /Cortex\/Loki managed alert/);
+    await waitFor(() => expect(mocks.api.discoverFeatures).toHaveBeenCalled());
+    await waitFor(() => expect(mocks.searchFolders).toHaveBeenCalled());
 
-    // wait for ui theck each datasource if it supports rule editing
-    await waitFor(() => expect(mocks.api.fetchRulerRulesGroup).toHaveBeenCalledTimes(4));
+    await ui.inputs.name.find();
+    await userEvent.click(await ui.buttons.lotexAlert.get());
 
     // check that only rules sources that have ruler available are there
     const dataSourceSelect = ui.inputs.dataSource.get();
-    userEvent.click(byRole('textbox').get(dataSourceSelect));
+    await userEvent.click(byRole('combobox').get(dataSourceSelect));
     expect(await byText('loki with ruler').query()).toBeInTheDocument();
     expect(byText('cortex with ruler').query()).toBeInTheDocument();
     expect(byText('loki with local rule store').query()).not.toBeInTheDocument();
@@ -501,6 +676,6 @@ describe('RuleEditor', () => {
 });
 
 const clickSelectOption = async (selectElement: HTMLElement, optionText: Matcher): Promise<void> => {
-  userEvent.click(byRole('textbox').get(selectElement));
+  await userEvent.click(byRole('combobox').get(selectElement));
   await selectOptionInTest(selectElement, optionText as string);
 };
