@@ -1,7 +1,7 @@
-import React from 'react';
 import { css } from '@emotion/css';
+import React from 'react';
+
 import { NavModel, NavModelItem } from '@grafana/data';
-import { Page as PluginPage } from '../components/Page';
 import { Page } from 'app/core/components/Page/Page';
 
 const node: NavModelItem = {
@@ -17,18 +17,16 @@ export default function NotEnabled(): JSX.Element | null {
   return (
     <Page navModel={navModel}>
       <Page.Contents>
-        <PluginPage>
-          To enable installing plugins via catalog, please refer to the{' '}
-          <a
-            className={css`
-              text-decoration: underline;
-            `}
-            href="https://grafana.com/docs/grafana/latest/plugins/catalog"
-          >
-            Plugin Catalog
-          </a>{' '}
-          instructions
-        </PluginPage>
+        To enable installing plugins via catalog, please refer to the{' '}
+        <a
+          className={css`
+            text-decoration: underline;
+          `}
+          href="https://grafana.com/docs/grafana/latest/plugins/catalog"
+        >
+          Plugin Catalog
+        </a>{' '}
+        instructions
       </Page.Contents>
     </Page>
   );
