@@ -1,17 +1,19 @@
 import React, { FC, useMemo } from 'react';
+
 import { SelectableValue } from '@grafana/data';
 import { Select } from '@grafana/ui';
-import { MetricQuery } from '../types';
+
 import { getAlignmentPickerData } from '../functions';
-import { SELECT_WIDTH } from '../constants';
+import { MetricQuery } from '../types';
 
 export interface Props {
+  inputId: string;
   onChange: (query: MetricQuery) => void;
   query: MetricQuery;
   templateVariableOptions: Array<SelectableValue<string>>;
 }
 
-export const AlignmentFunction: FC<Props> = ({ query, templateVariableOptions, onChange }) => {
+export const AlignmentFunction: FC<Props> = ({ inputId, query, templateVariableOptions, onChange }) => {
   const { valueType, metricKind, perSeriesAligner: psa, preprocessor } = query;
   const { perSeriesAligner, alignOptions } = useMemo(
     () => getAlignmentPickerData(valueType, metricKind, psa, preprocessor),
@@ -20,8 +22,6 @@ export const AlignmentFunction: FC<Props> = ({ query, templateVariableOptions, o
 
   return (
     <Select
-      menuShouldPortal
-      width={SELECT_WIDTH}
       onChange={({ value }) => onChange({ ...query, perSeriesAligner: value! })}
       value={[...alignOptions, ...templateVariableOptions].find((s) => s.value === perSeriesAligner)}
       options={[
@@ -36,6 +36,7 @@ export const AlignmentFunction: FC<Props> = ({ query, templateVariableOptions, o
         },
       ]}
       placeholder="Select Alignment"
-    ></Select>
+      inputId={inputId}
+    />
   );
 };
