@@ -1,8 +1,15 @@
+import { descending, deviation } from 'd3';
+import { flatten, forOwn, groupBy, partition } from 'lodash';
+
 import {
   ArrayDataFrame,
   ArrayVector,
+  CoreApp,
   DataFrame,
+  DataFrameType,
   DataLink,
+  DataQueryRequest,
+  DataQueryResponse,
   DataTopic,
   Field,
   FieldType,
@@ -10,12 +17,14 @@ import {
   getDisplayProcessor,
   Labels,
   MutableField,
+  PreferredVisualisationType,
   ScopedVars,
   TIME_SERIES_TIME_FIELD_NAME,
   TIME_SERIES_VALUE_FIELD_NAME,
 } from '@grafana/data';
 import { FetchResponse, getDataSourceSrv, getTemplateSrv } from '@grafana/runtime';
-import { descending, deviation } from 'd3';
+
+import { renderLegendFormat } from './legend';
 import {
   ExemplarTraceIdDestination,
   isExemplarData,
@@ -29,14 +38,189 @@ import {
   TransformOptions,
 } from './types';
 
-const POSITIVE_INFINITY_SAMPLE_VALUE = '+Inf';
-const NEGATIVE_INFINITY_SAMPLE_VALUE = '-Inf';
+// handles case-insensitive Inf, +Inf, -Inf (with optional "inity" suffix)
+const INFINITY_SAMPLE_REGEX = /^[+-]?inf(?:inity)?$/i;
 
 interface TimeAndValue {
   [TIME_SERIES_TIME_FIELD_NAME]: number;
   [TIME_SERIES_VALUE_FIELD_NAME]: number;
 }
 
+const isTableResult = (dataFrame: DataFrame, options: DataQueryRequest<PromQuery>): boolean => {
+  // We want to process vector and scalar results in Explore as table
+  if (
+    options.app === CoreApp.Explore &&
+    (dataFrame.meta?.custom?.resultType === 'vector' || dataFrame.meta?.custom?.resultType === 'scalar')
+  ) {
+    return true;
+  }
+
+  // We want to process all dataFrames with target.format === 'table' as table
+  const target = options.targets.find((target) => target.refId === dataFrame.refId);
+  return target?.format === 'table';
+};
+
+const isHeatmapResult = (dataFrame: DataFrame, options: DataQueryRequest<PromQuery>): boolean => {
+  const target = options.targets.find((target) => target.refId === dataFrame.refId);
+  return target?.format === 'heatmap';
+};
+
+// V2 result transformer used to transform query results from queries that were run through prometheus backend
+export function transformV2(
+  response: DataQueryResponse,
+  request: DataQueryRequest<PromQuery>,
+  options: { exemplarTraceIdDestinations?: ExemplarTraceIdDestination[] }
+) {
+  const [tableFrames, framesWithoutTable] = partition<DataFrame>(response.data, (df) => isTableResult(df, request));
+  const processedTableFrames = transformDFToTable(tableFrames);
+
+  const [exemplarFrames, framesWithoutTableAndExemplars] = partition<DataFrame>(
+    framesWithoutTable,
+    (df) => df.meta?.custom?.resultType === 'exemplar'
+  );
+
+  // EXEMPLAR FRAMES: We enrich exemplar frames with data links and add dataTopic meta info
+  const { exemplarTraceIdDestinations: destinations } = options;
+  const processedExemplarFrames = exemplarFrames.map((dataFrame) => {
+    if (destinations?.length) {
+      for (const exemplarTraceIdDestination of destinations) {
+        const traceIDField = dataFrame.fields.find((field) => field.name === exemplarTraceIdDestination.name);
+        if (traceIDField) {
+          const links = getDataLinks(exemplarTraceIdDestination);
+          traceIDField.config.links = traceIDField.config.links?.length
+            ? [...traceIDField.config.links, ...links]
+            : links;
+        }
+      }
+    }
+
+    return { ...dataFrame, meta: { ...dataFrame.meta, dataTopic: DataTopic.Annotations } };
+  });
+
+  const [heatmapResults, framesWithoutTableHeatmapsAndExemplars] = partition<DataFrame>(
+    framesWithoutTableAndExemplars,
+    (df) => isHeatmapResult(df, request)
+  );
+
+  // Group heatmaps by query
+  const heatmapResultsGroupedByQuery = groupBy<DataFrame>(heatmapResults, (h) => h.refId);
+
+  // Initialize empty array to push grouped histogram frames to
+  let processedHeatmapResultsGroupedByQuery: DataFrame[][] = [];
+
+  // Iterate through every query in this heatmap
+  for (const query in heatmapResultsGroupedByQuery) {
+    // Get reference to dataFrames for heatmap
+    const heatmapResultsGroup = heatmapResultsGroupedByQuery[query];
+
+    // Create a new grouping by iterating through the data frames...
+    const heatmapResultsGroupedByValues = groupBy<DataFrame>(heatmapResultsGroup, (dataFrame) => {
+      // Each data frame has `Time` and `Value` properties, we want to get the values
+      const values = dataFrame.fields.find((field) => field.name === TIME_SERIES_VALUE_FIELD_NAME);
+      // Specific functionality for special "le" quantile heatmap value, we know if this value exists, that we do not want to calculate the heatmap density across data frames from the same quartile
+      if (values?.labels && HISTOGRAM_QUANTILE_LABEL_NAME in values.labels) {
+        const { le, ...notLE } = values?.labels;
+        return Object.values(notLE).join();
+      }
+
+      // Return a string made from the concatenation of this frame's values to represent a grouping in the query
+      return Object.values(values?.labels ?? []).join();
+    });
+
+    // Then iterate through the resultant object
+    forOwn(heatmapResultsGroupedByValues, (dataFrames, key) => {
+      // Sort frames within each grouping
+      const sortedHeatmap = dataFrames.sort(sortSeriesByLabel);
+      // And push the sorted grouping with the rest
+      processedHeatmapResultsGroupedByQuery.push(mergeHeatmapFrames(transformToHistogramOverTime(sortedHeatmap)));
+    });
+  }
+
+  // Everything else is processed as time_series result and graph preferredVisualisationType
+  const otherFrames = framesWithoutTableHeatmapsAndExemplars.map((dataFrame) => {
+    const df = {
+      ...dataFrame,
+      meta: {
+        ...dataFrame.meta,
+        preferredVisualisationType: 'graph',
+      },
+    } as DataFrame;
+    return df;
+  });
+
+  const flattenedProcessedHeatmapFrames = flatten(processedHeatmapResultsGroupedByQuery);
+
+  return {
+    ...response,
+    data: [...otherFrames, ...processedTableFrames, ...flattenedProcessedHeatmapFrames, ...processedExemplarFrames],
+  };
+}
+
+const HISTOGRAM_QUANTILE_LABEL_NAME = 'le';
+
+export function transformDFToTable(dfs: DataFrame[]): DataFrame[] {
+  // If no dataFrames or if 1 dataFrames with no values, return original dataFrame
+  if (dfs.length === 0 || (dfs.length === 1 && dfs[0].length === 0)) {
+    return dfs;
+  }
+
+  // Group results by refId and process dataFrames with the same refId as 1 dataFrame
+  const dataFramesByRefId = groupBy(dfs, 'refId');
+  const refIds = Object.keys(dataFramesByRefId);
+
+  const frames = refIds.map((refId) => {
+    // Create timeField, valueField and labelFields
+    const valueText = getValueText(refIds.length, refId);
+    const valueField = getValueField({ data: [], valueName: valueText });
+    const timeField = getTimeField([]);
+    const labelFields: MutableField[] = [];
+
+    // Fill labelsFields with labels from dataFrames
+    dataFramesByRefId[refId].forEach((df) => {
+      const frameValueField = df.fields[1];
+      const promLabels = frameValueField.labels ?? {};
+
+      Object.keys(promLabels)
+        .sort()
+        .forEach((label) => {
+          // If we don't have label in labelFields, add it
+          if (!labelFields.some((l) => l.name === label)) {
+            const numberField = label === HISTOGRAM_QUANTILE_LABEL_NAME;
+            labelFields.push({
+              name: label,
+              config: { filterable: true },
+              type: numberField ? FieldType.number : FieldType.string,
+              values: new ArrayVector(),
+            });
+          }
+        });
+    });
+
+    // Fill valueField, timeField and labelFields with values
+    dataFramesByRefId[refId].forEach((df) => {
+      df.fields[0].values.toArray().forEach((value) => timeField.values.add(value));
+      df.fields[1].values.toArray().forEach((value) => {
+        valueField.values.add(parseSampleValue(value));
+        const labelsForField = df.fields[1].labels ?? {};
+        labelFields.forEach((field) => field.values.add(getLabelValue(labelsForField, field.name)));
+      });
+    });
+
+    const fields = [timeField, ...labelFields, valueField];
+    return {
+      refId,
+      fields,
+      meta: { ...dfs[0].meta, preferredVisualisationType: 'table' as PreferredVisualisationType },
+      length: timeField.values.length,
+    };
+  });
+  return frames;
+}
+
+function getValueText(responseLength: number, refId = '') {
+  return responseLength > 1 ? `Value #${refId}` : 'Value';
+}
+
 export function transform(
   response: FetchResponse<PromDataSuccessResponse>,
   transformOptions: {
@@ -89,7 +273,7 @@ export function transform(
     // Add data links if configured
     if (transformOptions.exemplarTraceIdDestinations?.length) {
       for (const exemplarTraceIdDestination of transformOptions.exemplarTraceIdDestinations) {
-        const traceIDField = dataFrame.fields.find((field) => field.name === exemplarTraceIdDestination!.name);
+        const traceIDField = dataFrame.fields.find((field) => field.name === exemplarTraceIdDestination.name);
         if (traceIDField) {
           const links = getDataLinks(exemplarTraceIdDestination);
           traceIDField.config.links = traceIDField.config.links?.length
@@ -129,9 +313,7 @@ export function transform(
 
   // When format is heatmap use the already created data frames and transform it more
   if (options.format === 'heatmap') {
-    dataFrame.sort(sortSeriesByLabel);
-    const seriesList = transformToHistogramOverTime(dataFrame);
-    return seriesList;
+    return mergeHeatmapFrames(transformToHistogramOverTime(dataFrame.sort(sortSeriesByLabel)));
   }
 
   // Return matrix or vector result as DataFrame[]
@@ -145,20 +327,26 @@ function getDataLinks(options: ExemplarTraceIdDestination): DataLink[] {
     const dataSourceSrv = getDataSourceSrv();
     const dsSettings = dataSourceSrv.getInstanceSettings(options.datasourceUid);
 
-    dataLinks.push({
-      title: `Query with ${dsSettings?.name}`,
-      url: '',
-      internal: {
-        query: { query: '${__value.raw}', queryType: 'traceId' },
-        datasourceUid: options.datasourceUid,
-        datasourceName: dsSettings?.name ?? 'Data source not found',
-      },
-    });
+    // dsSettings is undefined because of the reasons below:
+    // - permissions issues (probably most likely)
+    // - deleted datasource
+    // - misconfiguration
+    if (dsSettings) {
+      dataLinks.push({
+        title: options.urlDisplayLabel || `Query with ${dsSettings?.name}`,
+        url: '',
+        internal: {
+          query: { query: '${__value.raw}', queryType: 'traceId' },
+          datasourceUid: options.datasourceUid,
+          datasourceName: dsSettings?.name ?? 'Data source not found',
+        },
+      });
+    }
   }
 
   if (options.url) {
     dataLinks.push({
-      title: `Go to ${options.url}`,
+      title: options.urlDisplayLabel || `Go to ${options.url}`,
       url: options.url,
       targetBlank: true,
     });
@@ -287,7 +475,7 @@ function transformMetricDataToTable(md: MatrixOrVectorResult[], options: Transfo
     .map((label) => {
       // Labels have string field type, otherwise table tries to figure out the type which can result in unexpected results
       // Only "le" label has a number field type
-      const numberField = label === 'le';
+      const numberField = label === HISTOGRAM_QUANTILE_LABEL_NAME;
       return {
         name: label,
         config: { filterable: true },
@@ -321,7 +509,7 @@ function transformMetricDataToTable(md: MatrixOrVectorResult[], options: Transfo
 
 function getLabelValue(metric: PromMetric, label: string): string | number {
   if (metric.hasOwnProperty(label)) {
-    if (label === 'le') {
+    if (label === HISTOGRAM_QUANTILE_LABEL_NAME) {
       return parseSampleValue(metric[label]);
     }
     return metric[label];
@@ -337,6 +525,7 @@ function getTimeField(data: PromValue[], isMs = false): MutableField {
     values: new ArrayVector<number>(data.map((val) => (isMs ? val[0] : val[0] * 1000))),
   };
 }
+
 type ValueFieldOptions = {
   data: PromValue[];
   valueName?: string;
@@ -366,7 +555,7 @@ function getValueField({
 
 function createLabelInfo(labels: { [key: string]: string }, options: TransformOptions) {
   if (options?.legendFormat) {
-    const title = renderTemplate(getTemplateSrv().replace(options.legendFormat, options?.scopedVars), labels);
+    const title = renderLegendFormat(getTemplateSrv().replace(options.legendFormat, options?.scopedVars), labels);
     return { name: title, labels };
   }
 
@@ -390,14 +579,31 @@ export function getOriginalMetricName(labelData: { [key: string]: string }) {
   return `${metricName}{${labelPart}}`;
 }
 
-export function renderTemplate(aliasPattern: string, aliasData: { [key: string]: string }) {
-  const aliasRegex = /\{\{\s*(.+?)\s*\}\}/g;
-  return aliasPattern.replace(aliasRegex, (_match, g1) => {
-    if (aliasData[g1]) {
-      return aliasData[g1];
-    }
-    return '';
+function mergeHeatmapFrames(frames: DataFrame[]): DataFrame[] {
+  if (frames.length === 0) {
+    return [];
+  }
+
+  const timeField = frames[0].fields.find((field) => field.type === FieldType.time)!;
+  const countFields = frames.map((frame) => {
+    let field = frame.fields.find((field) => field.type === FieldType.number)!;
+
+    return {
+      ...field,
+      name: field.config.displayNameFromDS!,
+    };
   });
+
+  return [
+    {
+      ...frames[0],
+      meta: {
+        ...frames[0].meta,
+        type: DataFrameType.HeatmapRows,
+      },
+      fields: [timeField!, ...countFields],
+    },
+  ];
 }
 
 function transformToHistogramOverTime(seriesList: DataFrame[]) {
@@ -446,13 +652,10 @@ function sortSeriesByLabel(s1: DataFrame, s2: DataFrame): number {
   return 0;
 }
 
-function parseSampleValue(value: string): number {
-  switch (value) {
-    case POSITIVE_INFINITY_SAMPLE_VALUE:
-      return Number.POSITIVE_INFINITY;
-    case NEGATIVE_INFINITY_SAMPLE_VALUE:
-      return Number.NEGATIVE_INFINITY;
-    default:
-      return parseFloat(value);
+/** @internal */
+export function parseSampleValue(value: string): number {
+  if (INFINITY_SAMPLE_REGEX.test(value)) {
+    return value[0] === '-' ? Number.NEGATIVE_INFINITY : Number.POSITIVE_INFINITY;
   }
+  return parseFloat(value);
 }
